/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import TableView from '../manageOwnOrGroupExerciseComponents/listView'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import Spinner from "../../helpers/spinner";
import {
  loadManageGroupExercisesSite,
  setActivePaginationPage, setIsLoading,
  setPaginationPageSize,
  setSearchText,
  setSortByKey
} from "../../../state/actions/manageGroupExercisesSite/groupExercisesSiteActions";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import TagFilterPanel from '../../tagComponents/tagsFilterPanel'
import {InputOnChangeEvent} from '../../../types/reactEvents'
import {
  deleteGroupExercisesAsync,
  getGroupExercisesAsync
} from '../../../state/actions/manageGroupExercisesSite/groupExercisesCrudSiteActions'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groups: rootState.manageGroupExercisesState.groups,
    groupExercises: rootState.manageGroupExercisesState.groupExercises,
    isLoading: rootState.manageGroupExercisesState.isLoading
      || rootState.manageTagsState.isLoading,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    tags: rootState.manageTagsState.tags,
    langId: rootState.i18nState.langId,

    isTagsFilterDisplayed: rootState.manageGroupExercisesState.isTagsFilterDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadManageGroupExercisesSite,
                                                                        setSortByKey,
                                                                        deleteGroupExercisesAsync,
                                                                        setSearchText,
                                                                        setPaginationPageSize,
                                                                        setActivePaginationPage,
                                                                        getGroupExercisesAsync,
                                                                        setIsLoading,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupExercisesSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {

    try {
      await this.props.loadManageGroupExercisesSite()
    } catch (err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {
    return (

      <div className="multiple-single-panels">

        {
          //we now display TagFilterPanel that can show the loading...
          //BUT is could be hidden...
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isTagsFilterDisplayed &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
            onTagsUpdated={async () => {
              // noinspection TsLint
              for (let i = 0; i < this.props.groups.length; i++) {
                //maybe we don't need the await... this would speed things up..
                //without await it's a race condition?
                //if we start both requests and the 1. finishes and transforms the state...
                //during this the second finishes but does the transform on the old state...
                //then the results from the first are swallowed
                await this.props.getGroupExercisesAsync(this.props.groups[i].id)
              }
            }}
          />
        }


        {
          //foreach
          //via tag search remove groups with no hits... but keep the ones where we entered search text
          //if we entered search text then don't hide the group (else we would hide the searched group... and we can no longer access the search input)
          this.props.groupExercises.filter(
            p => p.exercisePreviews.length > 0 || p.pagination.searchText.trim() !== '').map((groupTuple, index) => {
            return (
              <SinglePanelSiteWrapper key={groupTuple.userGroup.id}>
                <div className="view-padding">
                  <div className="view-head-line">
                    <div className="view-caption">
                      <h1>
                        {groupTuple.userGroup.displayName}
                      </h1>
                    </div>
                    <div className="view-options">
                      <MaterialSearchInput
                        value={groupTuple.pagination.searchText}
                        onChange={(e: InputOnChangeEvent) => {
                          this.props.setSearchText(e.currentTarget.value, groupTuple.userGroup.id)
                        }}
                      />
                    </div>
                  </div>
                  <div className="view-content">
                    <TableView
                      isLoading={groupTuple.isLoading}
                      groupId={groupTuple.userGroup.id}
                      exercisePreviews={groupTuple.exercisePreviews}
                      comesFromOwnExercisesSite={false}
                      sortByKey={groupTuple.pagination.orderByProp}
                      sortDirection={groupTuple.pagination.isOrderByDesc}
                      setSortByKey={this.props.setSortByKey}
                      setPaginationPageSize={this.props.setPaginationPageSize}
                      setActivePaginationPage={this.props.setActivePaginationPage}
                      pagination={groupTuple.pagination}
                      onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                        this.props.deleteGroupExercisesAsync(exercisePreview.id, groupTuple.userGroup.id)
                      }}
                      currentUserGroupRole={groupTuple.userGroup.groupRole}
                      setIsLoading={this.props.setIsLoading}
                    />

                  </div>
                </div>


              </SinglePanelSiteWrapper>
            )
          })
        }

      </div>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupExercisesSite)
